# -*- coding: utf-8 -*-

class _ServicePrincipalMixin:
    @classmethod
    def a4b(cls): return cls._build("a4b.amazonaws.com") # pragma: no cover

    @classmethod
    def acm_pca(cls): return cls._build("acm-pca.amazonaws.com") # pragma: no cover

    @classmethod
    def acm(cls): return cls._build("acm.amazonaws.com") # pragma: no cover

    @classmethod
    def alexa_app(cls): return cls._build("alexa-app.amazonaws.com") # pragma: no cover

    @classmethod
    def alexa_connectedh(cls): return cls._build("alexa-connectedh.amazonaws.com") # pragma: no cover

    @classmethod
    def amazonmq(cls): return cls._build("amazonmq.amazonaws.com") # pragma: no cover

    @classmethod
    def apigateway(cls): return cls._build("apigateway.amazonaws.com") # pragma: no cover

    @classmethod
    def appflow(cls): return cls._build("appflow.amazonaws.com") # pragma: no cover

    @classmethod
    def application_autoscaling(cls): return cls._build("application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def appstream_application_autoscaling(cls): return cls._build("appstream.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def appsync(cls): return cls._build("appsync.amazonaws.com") # pragma: no cover

    @classmethod
    def athena(cls): return cls._build("athena.amazonaws.com") # pragma: no cover

    @classmethod
    def autoscaling(cls): return cls._build("autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def aws_artifact_account_sync(cls): return cls._build("aws-artifact-account-sync.amazonaws.com") # pragma: no cover

    @classmethod
    def backup(cls): return cls._build("backup.amazonaws.com") # pragma: no cover

    @classmethod
    def batch(cls): return cls._build("batch.amazonaws.com") # pragma: no cover

    @classmethod
    def billingconsole(cls): return cls._build("billingconsole.amazonaws.com") # pragma: no cover

    @classmethod
    def budgets(cls): return cls._build("budgets.amazonaws.com") # pragma: no cover

    @classmethod
    def ce(cls): return cls._build("ce.amazonaws.com") # pragma: no cover

    @classmethod
    def channels_lex(cls): return cls._build("channels.lex.amazonaws.com") # pragma: no cover

    @classmethod
    def chime(cls): return cls._build("chime.amazonaws.com") # pragma: no cover

    @classmethod
    def cloud9(cls): return cls._build("cloud9.amazonaws.com") # pragma: no cover

    @classmethod
    def clouddirectory(cls): return cls._build("clouddirectory.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudformation(cls): return cls._build("cloudformation.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudfront(cls): return cls._build("cloudfront.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudhsm(cls): return cls._build("cloudhsm.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudsearch(cls): return cls._build("cloudsearch.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudtrail(cls): return cls._build("cloudtrail.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudwatch_crossaccount(cls): return cls._build("cloudwatch-crossaccount.amazonaws.com") # pragma: no cover

    @classmethod
    def codebuild(cls): return cls._build("codebuild.amazonaws.com") # pragma: no cover

    @classmethod
    def codecommit(cls): return cls._build("codecommit.amazonaws.com") # pragma: no cover

    @classmethod
    def codedeploy(cls): return cls._build("codedeploy.${AWS::Region}.amazonaws.com") # pragma: no cover

    @classmethod
    def codedeploy(cls): return cls._build("codedeploy.amazonaws.com") # pragma: no cover

    @classmethod
    def codepipeline(cls): return cls._build("codepipeline.amazonaws.com") # pragma: no cover

    @classmethod
    def codestar(cls): return cls._build("codestar.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_identity(cls): return cls._build("cognito-identity.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_idp(cls): return cls._build("cognito-idp.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_sync(cls): return cls._build("cognito-sync.amazonaws.com") # pragma: no cover

    @classmethod
    def config_conforms(cls): return cls._build("config-conforms.amazonaws.com") # pragma: no cover

    @classmethod
    def config_multiaccountsetup(cls): return cls._build("config-multiaccountsetup.amazonaws.com") # pragma: no cover

    @classmethod
    def config(cls): return cls._build("config.amazonaws.com") # pragma: no cover

    @classmethod
    def connect(cls): return cls._build("connect.amazonaws.com") # pragma: no cover

    @classmethod
    def continuousexport_discovery(cls): return cls._build("continuousexport.discovery.amazonaws.com") # pragma: no cover

    @classmethod
    def custom_resource_application_autoscaling(cls): return cls._build("custom-resource.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def databrew(cls): return cls._build("databrew.amazonaws.com") # pragma: no cover

    @classmethod
    def datapipeline(cls): return cls._build("datapipeline.amazonaws.com") # pragma: no cover

    @classmethod
    def dax(cls): return cls._build("dax.amazonaws.com") # pragma: no cover

    @classmethod
    def deeplens(cls): return cls._build("deeplens.amazonaws.com") # pragma: no cover

    @classmethod
    def delivery_logs(cls): return cls._build("delivery.logs.amazonaws.com") # pragma: no cover

    @classmethod
    def diode(cls): return cls._build("diode.amazonaws.com") # pragma: no cover

    @classmethod
    def directconnect(cls): return cls._build("directconnect.amazonaws.com") # pragma: no cover

    @classmethod
    def discovery(cls): return cls._build("discovery.amazonaws.com") # pragma: no cover

    @classmethod
    def dlm(cls): return cls._build("dlm.amazonaws.com") # pragma: no cover

    @classmethod
    def dms(cls): return cls._build("dms.amazonaws.com") # pragma: no cover

    @classmethod
    def ds(cls): return cls._build("ds.amazonaws.com") # pragma: no cover

    @classmethod
    def dynamodb(cls): return cls._build("dynamodb.amazonaws.com") # pragma: no cover

    @classmethod
    def dynamodb_application_autoscaling(cls): return cls._build("dynamodb.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def ec(cls): return cls._build("ec.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2(cls): return cls._build("ec2.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2_application_autoscaling(cls): return cls._build("ec2.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2fleet(cls): return cls._build("ec2fleet.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2scheduled(cls): return cls._build("ec2scheduled.amazonaws.com") # pragma: no cover

    @classmethod
    def ecr(cls): return cls._build("ecr.amazonaws.com") # pragma: no cover

    @classmethod
    def ecs_tasks(cls): return cls._build("ecs-tasks.amazonaws.com") # pragma: no cover

    @classmethod
    def ecs(cls): return cls._build("ecs.amazonaws.com") # pragma: no cover

    @classmethod
    def ecs_application_autoscaling(cls): return cls._build("ecs.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def edgelambda(cls): return cls._build("edgelambda.amazonaws.com") # pragma: no cover

    @classmethod
    def eks(cls): return cls._build("eks.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticache(cls): return cls._build("elasticache.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticbeanstalk(cls): return cls._build("elasticbeanstalk.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticfilesystem(cls): return cls._build("elasticfilesystem.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticloadbalancing(cls): return cls._build("elasticloadbalancing.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticmapreduce(cls): return cls._build("elasticmapreduce.amazonaws.com") # pragma: no cover

    @classmethod
    def elastictranscoder(cls): return cls._build("elastictranscoder.amazonaws.com") # pragma: no cover

    @classmethod
    def email_cognito_idp(cls): return cls._build("email.cognito-idp.amazonaws.com") # pragma: no cover

    @classmethod
    def es(cls): return cls._build("es.amazonaws.com") # pragma: no cover

    @classmethod
    def events(cls): return cls._build("events.amazonaws.com") # pragma: no cover

    @classmethod
    def firehose(cls): return cls._build("firehose.amazonaws.com") # pragma: no cover

    @classmethod
    def fms(cls): return cls._build("fms.amazonaws.com") # pragma: no cover

    @classmethod
    def freertos(cls): return cls._build("freertos.amazonaws.com") # pragma: no cover

    @classmethod
    def fsx(cls): return cls._build("fsx.amazonaws.com") # pragma: no cover

    @classmethod
    def gamelift(cls): return cls._build("gamelift.amazonaws.com") # pragma: no cover

    @classmethod
    def glacier(cls): return cls._build("glacier.amazonaws.com") # pragma: no cover

    @classmethod
    def globalaccelerator(cls): return cls._build("globalaccelerator.amazonaws.com") # pragma: no cover

    @classmethod
    def glue(cls): return cls._build("glue.amazonaws.com") # pragma: no cover

    @classmethod
    def greengrass(cls): return cls._build("greengrass.amazonaws.com") # pragma: no cover

    @classmethod
    def guardduty(cls): return cls._build("guardduty.amazonaws.com") # pragma: no cover

    @classmethod
    def health(cls): return cls._build("health.amazonaws.com") # pragma: no cover

    @classmethod
    def iam(cls): return cls._build("iam.amazonaws.com") # pragma: no cover

    @classmethod
    def importexport(cls): return cls._build("importexport.amazonaws.com") # pragma: no cover

    @classmethod
    def inspector(cls): return cls._build("inspector.amazonaws.com") # pragma: no cover

    @classmethod
    def iot(cls): return cls._build("iot.amazonaws.com") # pragma: no cover

    @classmethod
    def iotanalytics(cls): return cls._build("iotanalytics.amazonaws.com") # pragma: no cover

    @classmethod
    def iotevents(cls): return cls._build("iotevents.amazonaws.com") # pragma: no cover

    @classmethod
    def iotsitewise(cls): return cls._build("iotsitewise.amazonaws.com") # pragma: no cover

    @classmethod
    def iotthingsgraph(cls): return cls._build("iotthingsgraph.amazonaws.com") # pragma: no cover

    @classmethod
    def jellyfish(cls): return cls._build("jellyfish.amazonaws.com") # pragma: no cover

    @classmethod
    def kafka(cls): return cls._build("kafka.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesis(cls): return cls._build("kinesis.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesisanalytics(cls): return cls._build("kinesisanalytics.amazonaws.com") # pragma: no cover

    @classmethod
    def kms(cls): return cls._build("kms.amazonaws.com") # pragma: no cover

    @classmethod
    def lakeformation(cls): return cls._build("lakeformation.amazonaws.com") # pragma: no cover

    @classmethod
    def awslambda(cls): return cls._build("lambda.amazonaws.com") # pragma: no cover

    @classmethod
    def lex(cls): return cls._build("lex.amazonaws.com") # pragma: no cover

    @classmethod
    def license_manager(cls): return cls._build("license-manager.amazonaws.com") # pragma: no cover

    @classmethod
    def lightsail(cls): return cls._build("lightsail.amazonaws.com") # pragma: no cover

    @classmethod
    def logger_cloudfront(cls): return cls._build("logger.cloudfront.amazonaws.com") # pragma: no cover

    @classmethod
    def logs(cls): return cls._build("logs.amazonaws.com") # pragma: no cover

    @classmethod
    def machinelearning(cls): return cls._build("machinelearning.amazonaws.com") # pragma: no cover

    @classmethod
    def macie(cls): return cls._build("macie.amazonaws.com") # pragma: no cover

    @classmethod
    def managedservices(cls): return cls._build("managedservices.amazonaws.com") # pragma: no cover

    @classmethod
    def mediaconnect(cls): return cls._build("mediaconnect.amazonaws.com") # pragma: no cover

    @classmethod
    def mediaconvert(cls): return cls._build("mediaconvert.amazonaws.com") # pragma: no cover

    @classmethod
    def mediapackage(cls): return cls._build("mediapackage.amazonaws.com") # pragma: no cover

    @classmethod
    def mediastore(cls): return cls._build("mediastore.amazonaws.com") # pragma: no cover

    @classmethod
    def mediatailor(cls): return cls._build("mediatailor.amazonaws.com") # pragma: no cover

    @classmethod
    def member_org_stacksets_cloudformation(cls): return cls._build("member.org.stacksets.cloudformation.amazonaws.com") # pragma: no cover

    @classmethod
    def metering_marketplace(cls): return cls._build("metering-marketplace.amazonaws.com") # pragma: no cover

    @classmethod
    def migrationhub(cls): return cls._build("migrationhub.amazonaws.com") # pragma: no cover

    @classmethod
    def mobilehub(cls): return cls._build("mobilehub.amazonaws.com") # pragma: no cover

    @classmethod
    def monitoring(cls): return cls._build("monitoring.amazonaws.com") # pragma: no cover

    @classmethod
    def monitoring_rds(cls): return cls._build("monitoring.rds.amazonaws.com") # pragma: no cover

    @classmethod
    def ops_apigateway(cls): return cls._build("ops.apigateway.amazonaws.com") # pragma: no cover

    @classmethod
    def opsworks_cm(cls): return cls._build("opsworks-cm.amazonaws.com") # pragma: no cover

    @classmethod
    def opsworks(cls): return cls._build("opsworks.amazonaws.com") # pragma: no cover

    @classmethod
    def organizations(cls): return cls._build("organizations.amazonaws.com") # pragma: no cover

    @classmethod
    def pinpoint(cls): return cls._build("pinpoint.amazonaws.com") # pragma: no cover

    @classmethod
    def polly(cls): return cls._build("polly.amazonaws.com") # pragma: no cover

    @classmethod
    def qldb(cls): return cls._build("qldb.amazonaws.com") # pragma: no cover

    @classmethod
    def quicksight(cls): return cls._build("quicksight.amazonaws.com") # pragma: no cover

    @classmethod
    def ram(cls): return cls._build("ram.amazonaws.com") # pragma: no cover

    @classmethod
    def rds(cls): return cls._build("rds.amazonaws.com") # pragma: no cover

    @classmethod
    def redshift(cls): return cls._build("redshift.amazonaws.com") # pragma: no cover

    @classmethod
    def rekognition(cls): return cls._build("rekognition.amazonaws.com") # pragma: no cover

    @classmethod
    def replication_dynamodb(cls): return cls._build("replication.dynamodb.amazonaws.com") # pragma: no cover

    @classmethod
    def replicator_lambda(cls): return cls._build("replicator.lambda.amazonaws.com") # pragma: no cover

    @classmethod
    def resource_groups(cls): return cls._build("resource-groups.amazonaws.com") # pragma: no cover

    @classmethod
    def robomaker(cls): return cls._build("robomaker.amazonaws.com") # pragma: no cover

    @classmethod
    def route53(cls): return cls._build("route53.amazonaws.com") # pragma: no cover

    @classmethod
    def route53domains(cls): return cls._build("route53domains.amazonaws.com") # pragma: no cover

    @classmethod
    def route53resolver(cls): return cls._build("route53resolver.amazonaws.com") # pragma: no cover

    @classmethod
    def s3(cls): return cls._build("s3.amazonaws.com") # pragma: no cover

    @classmethod
    def sagemaker(cls): return cls._build("sagemaker.amazonaws.com") # pragma: no cover

    @classmethod
    def secretsmanager(cls): return cls._build("secretsmanager.amazonaws.com") # pragma: no cover

    @classmethod
    def securityhub(cls): return cls._build("securityhub.amazonaws.com") # pragma: no cover

    @classmethod
    def serverlessrepo(cls): return cls._build("serverlessrepo.amazonaws.com") # pragma: no cover

    @classmethod
    def servicecatalog(cls): return cls._build("servicecatalog.amazonaws.com") # pragma: no cover

    @classmethod
    def servicediscovery(cls): return cls._build("servicediscovery.amazonaws.com") # pragma: no cover

    @classmethod
    def ses(cls): return cls._build("ses.amazonaws.com") # pragma: no cover

    @classmethod
    def shield(cls): return cls._build("shield.amazonaws.com") # pragma: no cover

    @classmethod
    def signer(cls): return cls._build("signer.amazonaws.com") # pragma: no cover

    @classmethod
    def signin(cls): return cls._build("signin.amazonaws.com") # pragma: no cover

    @classmethod
    def sms(cls): return cls._build("sms.amazonaws.com") # pragma: no cover

    @classmethod
    def sns(cls): return cls._build("sns.amazonaws.com") # pragma: no cover

    @classmethod
    def spotfleet(cls): return cls._build("spotfleet.amazonaws.com") # pragma: no cover

    @classmethod
    def sqs(cls): return cls._build("sqs.amazonaws.com") # pragma: no cover

    @classmethod
    def ssm(cls): return cls._build("ssm.amazonaws.com") # pragma: no cover

    @classmethod
    def sso(cls): return cls._build("sso.amazonaws.com") # pragma: no cover

    @classmethod
    def states(cls): return cls._build("states.amazonaws.com") # pragma: no cover

    @classmethod
    def storagegateway(cls): return cls._build("storagegateway.amazonaws.com") # pragma: no cover

    @classmethod
    def sts(cls): return cls._build("sts.amazonaws.com") # pragma: no cover

    @classmethod
    def support(cls): return cls._build("support.amazonaws.com") # pragma: no cover

    @classmethod
    def swf(cls): return cls._build("swf.amazonaws.com") # pragma: no cover

    @classmethod
    def tagging(cls): return cls._build("tagging.amazonaws.com") # pragma: no cover

    @classmethod
    def tagpolicies_tag(cls): return cls._build("tagpolicies.tag.amazonaws.com") # pragma: no cover

    @classmethod
    def transcribe(cls): return cls._build("transcribe.amazonaws.com") # pragma: no cover

    @classmethod
    def transfer(cls): return cls._build("transfer.amazonaws.com") # pragma: no cover

    @classmethod
    def translate(cls): return cls._build("translate.amazonaws.com") # pragma: no cover

    @classmethod
    def trustedadvisor(cls): return cls._build("trustedadvisor.amazonaws.com") # pragma: no cover

    @classmethod
    def tts(cls): return cls._build("tts.amazonaws.com") # pragma: no cover

    @classmethod
    def vmie(cls): return cls._build("vmie.amazonaws.com") # pragma: no cover

    @classmethod
    def waf_regional(cls): return cls._build("waf-regional.amazonaws.com") # pragma: no cover

    @classmethod
    def waf(cls): return cls._build("waf.amazonaws.com") # pragma: no cover

    @classmethod
    def workdocs(cls): return cls._build("workdocs.amazonaws.com") # pragma: no cover

    @classmethod
    def worklink(cls): return cls._build("worklink.amazonaws.com") # pragma: no cover

    @classmethod
    def workmail(cls): return cls._build("workmail.amazonaws.com") # pragma: no cover

    @classmethod
    def workspaces(cls): return cls._build("workspaces.amazonaws.com") # pragma: no cover

    @classmethod
    def xray(cls): return cls._build("xray.amazonaws.com") # pragma: no cover
