# -*- coding: utf-8 -*-

from . import (
    appmesh,
    wafregional,
    quicksight,
    elasticache,
    appflow,
    lookoutmetrics,
    greengrass,
    athena,
    glue,
    sagemaker,
    s3,
    elasticbeanstalk,
    pinpoint,
    eventschemas,
    customerprofiles,
    apprunner,
    iotanalytics,
    dms,
    cloudtrail,
    xray,
    ses,
    pinpointemail,
    iot,
    autoscaling,
    cloudfront,
    emr,
    cassandra,
    wafv2,
    mediapackage,
    acmpca,
    iotwireless,
    kinesisanalyticsv2,
    cognito,
    ecs,
    kinesisanalytics,
    config,
    opsworks,
    batch,
    databrew,
    applicationautoscaling,
    ec2,
    imagebuilder,
    appsync,
    directoryservice,
    medialive,
    awslambda,
    elasticsearch,
    groundstation,
    iotevents,
    greengrassv2,
    managedblockchain,
    waf,
    networkfirewall,
    datasync,
    apigateway,
    appconfig,
    devopsguru,
    ecr,
    resourcegroups,
    efs,
    stepfunctions,
    kinesisfirehose,
    appstream,
    elasticloadbalancing,
    backup,
    gamelift,
    rds,
    dynamodb,
    eks,
    dlm,
    guardduty,
    cloudwatch,
    ssm,
    elasticloadbalancingv2,
    budgets,
    mediaconnect,
    codestar,
    ssmcontacts,
    auditmanager,
    codedeploy,
    lakeformation,
    networkmanager,
    route53,
    logs,
    s3outposts,
    applicationinsights,
    secretsmanager,
    codebuild,
    iotsitewise,
    cloudformation,
    ssmincidents,
    route53resolver,
    sns,
    apigatewayv2,
    codepipeline,
    msk,
    nimblestudio,
    globalaccelerator,
    ivs,
    amazonmq,
    datapipeline,
    fis,
    iam,
    frauddetector,
    servicecatalog,
    location,
    licensemanager,
    mediastore,
    events,
    signer,
    kendra,
    amplify,
    robomaker,
    macie,
    redshift,
    synthetics,
    autoscalingplans,
    ask,
    certificatemanager,
    servicediscovery,
    accessanalyzer,
    s3objectlambda,
    iotthingsgraph,
    qldb,
    mediaconvert,
    ce,
    appintegrations,
    fms,
    kinesis,
    codecommit,
    emrcontainers,
    neptune,
    sso,
    fsx,
    finspace,
    workspaces,
    transfer,
    mwaa,
    iot1click,
    dax,
    codeguruprofiler,
    opsworkscm,
    codestarnotifications,
    cloud9,
    ram,
    sqs,
    codestarconnections,
    kms,
    iotfleethub,
    codeartifact,
    docdb,
    servicecatalogappregistry,
    detective,
    sdb,
    codegurureviewer,
    iotcoredeviceadvisor,
    timestream,
    inspector,
    securityhub,
    cur,
    lookoutvision,
    chatbot,
)
